#-----------------------------------------------------------------------------------------------
# DEO'S TOOL
#-----------------------------------------------------------------------------------------------

# region ADD-ON

bl_info = {
  "category": "Deo's",
  "name": "Deo's Tools Kit",
  # "name": "Deo's Tools Optimize",
  "description": "Optimize Meshes and Textures",
  "author": "Deodato Pechir",
  "blender": (4, 0, 0),
  "location": "3D View > Sidebars > Item > Deo's Tools",
  "tracker_url": "http://deodatopr.github.io/deostools",
}

# endregion

# region IMPORTS

# [Blender]
import bpy, os

# [Types]
from bpy.types import (Panel, Operator, PropertyGroup)

# [Props (UI)]
from bpy.props import (BoolProperty, FloatProperty, IntProperty, StringProperty, EnumProperty, CollectionProperty, PointerProperty)

# [Utilities (Icons)]
from bpy.utils import previews

#endregion


#-----------------------------------------------------------------------------------------------
# VARIABLES >> GLOBALS
#-----------------------------------------------------------------------------------------------

# region Paths
# pathAsset = bpy.utils.resource_path('USER') + "\\scripts\\addons\\deos_Optimize\\Name.ext"  # Blender Root
# pathAsset = os.path.dirname (__file__) + "Name.ext"                                         # Addon Root
# endregion

# region IconLibrary

iconInst = bpy.utils.previews.new()
iconsPath = os.path.dirname(__file__) + "\\UI\\"
iconInst.load("MyIconTris_1", os.path.join(iconsPath, "Ico_tris_1.png"), 'IMAGE')
iconInst.load("MyIconTris_2", os.path.join(iconsPath, "Ico_tris_2.png"), 'IMAGE')
iconInst.load("MyIconTris_3", os.path.join(iconsPath, "Ico_tris_3.png"), 'IMAGE')
iconInst.load("MyIconTris_4", os.path.join(iconsPath, "Ico_tris_4.png"), 'IMAGE')
iconInst.load("MyIconTris_5", os.path.join(iconsPath, "Ico_tris_5.png"), 'IMAGE')
iconInst.load("MyIconTris_6", os.path.join(iconsPath, "Ico_tris_6.png"), 'IMAGE')
iconInst.load("MyIconTris_7", os.path.join(iconsPath, "Ico_tris_7.png"), 'IMAGE')
iconInst.load("MyIconTris_8", os.path.join(iconsPath, "Ico_tris_8.png"), 'IMAGE')

# endregion


#-----------------------------------------------------------------------------------------------
# PROPERTIES
#-----------------------------------------------------------------------------------------------

class DeosPropertyGroupOptimize (PropertyGroup):

# region Visualize

  enum_OptVisCol_SelectMode : EnumProperty (
    name = "",
    description = "",
    items = 
      [ 
        ('Op1', "Complete", "All meshes in the scene"), 
        ('Op2', "Selected", "From the selected meshes"), 
        ('Op3', "Collection", "From the loaded collection"), 
      ],
    default = 'Op1',
  ) # type: ignore

  picker_OptVisCol_Collect : PointerProperty (
    name = "",
    description = "Load the Collection to analize",
    type = bpy.types.Collection,
  ) # type: ignore
  
  str_OptVisCol_ObjAnalized : StringProperty (
    name="",
    description="",
    default="0",
    maxlen=1024
  ) # type: ignore

  str_OptVisCol_MinTris : StringProperty (
    name="",
    description="",
    default="0",
    maxlen=1024
  ) # type: ignore

  str_OptVisCol_MaxTris : StringProperty (
    name="",
    description="",
    default="0",
    maxlen=1024
  ) # type: ignore

  enum_OptVisCol_Colors : EnumProperty (
    name = "",
    description = "",
    items = 
      [ 
        # Val, Name, Desc, Icon, Index
        ('Op1', "", "", iconInst["MyIconTris_1"].icon_id, 0),
        ('Op2', "", "", iconInst["MyIconTris_2"].icon_id, 0),
        ('Op3', "", "", iconInst["MyIconTris_3"].icon_id, 0),
        ('Op4', "", "", iconInst["MyIconTris_4"].icon_id, 0),
        ('Op5', "", "", iconInst["MyIconTris_5"].icon_id, 0),
        ('Op6', "", "", iconInst["MyIconTris_6"].icon_id, 0),
        ('Op7', "", "", iconInst["MyIconTris_7"].icon_id, 0),
        ('Op8', "", "", iconInst["MyIconTris_8"].icon_id, 0),
      ],
    default = 'Op1',
  ) # type: ignore

# endregion


# region PolyCount

  enum_OptPlyCnt_SelectMode : EnumProperty (
    name = "",
    description = "",
    items = 
      [ 
        ('Op1', "Complete", "All meshes in the scene"), 
        ('Op2', "Selected", "From the selected meshes"), 
        ('Op3', "Collection", "From the loaded collection"), 
      ],
    default = 'Op1',
  ) # type: ignore
  
  picker_OptPlyCnt_Collect : PointerProperty (
    name = "",
    description = "Load the Collection to analize",
    type = bpy.types.Collection,
  ) # type: ignore
  
  enum_OptPlyCnt_Action : EnumProperty (
    name = "",
    description = "",
    items = 
      [ 
        ('Op1', "<", "Tris amount Less than..."),
        ('Op2', ">", "Tris amount Grather than..."), 
        ('Op3', "=", "Tris amount Equal to.."), 
      ],
    default = 'Op1',
  ) # type: ignore
  
  int_OptPlyCnt_Amount : IntProperty (
    name="",
    description="",
    default=0,
  ) # type: ignore

  str_OptPlyCnt_ObjSel : StringProperty (
    name="",
    description="",
    default="00,000",
    maxlen=1024
  ) # type: ignore


# endregion


#-----------------------------------------------------------------------------------------------
# INTERFACE
#-----------------------------------------------------------------------------------------------

# Header 
class DeoPanelOptimizeHeader (Panel) :
  bl_idname = "DEOS_PT_Optimize_Header"
  bl_label = "DEO's OPTIMIZER SCENE"
  bl_category = "Deo's Tools"
  
  bl_context = ".objectmode"
  bl_space_type = "VIEW_3D"
  bl_region_type = "UI"

  def draw(self, context) :
    pass

# Meshes - Visualize
class DeoPanelOptimizeMeshVisualize (Panel) :
  bl_parent_id = "DEOS_PT_Optimize_Header"

  bl_idname = "DEOS_PT_Optimize_VisualColors"
  bl_label = "[ MESH ]    : : VISUAL COLORS : :"
  bl_space_type = "VIEW_3D"
  bl_region_type = "UI"
  bl_options = {'DEFAULT_CLOSED'}
  
  def draw(self, context) :
    OptimizePropGrps = context.scene.optimizePropertyGrp
    layout = self.layout
    box = layout.box()
    rowSize = 0.5

  #region Calculate 

    # Selection Mode
    row = box.row()
    row.scale_y = 1.2
    row.prop(OptimizePropGrps, "enum_OptVisCol_SelectMode", text=" ", expand=True)             # Buttons
    # row.prop(OptimizePropGrps, "enum_OptVisCol_SelectMode", text="Mode", icon="OBJECT_DATA") # Dropdown
    
    # Collection Visible
    if OptimizePropGrps.enum_OptVisCol_SelectMode == 'Op3' :
      box.prop(OptimizePropGrps, "picker_OptVisCol_Collect", text="", icon="OUTLINER_COLLECTION")

    # Buttons
    split = box.column().split(factor = 0.8)
    split.operator ("deo.operator_empty", text="V I S U A L I Z E")
    split.operator ("deo.operator_empty", text="", icon="TRASH")
  
  # endregion

  #region Info 

    row = box.row()
    row.alignment = 'CENTER'
    row.label(text="[   I n f o r m a t i o n   ]")


    # Obj Analized
    row = box.row().split(factor = rowSize)
    row.label(text="# Obj Analized:")
    row.prop (OptimizePropGrps, "str_OptVisCol_ObjAnalized", text="", emboss=False)


    
    # Tris
    row = box.row().split(factor = rowSize)
    row.label(text="Min Mesh Tris:")
    row.prop (OptimizePropGrps, "str_OptVisCol_MinTris", text="", emboss=False)
    
    row = box.column().split(factor = rowSize)
    row.label(text="Max Mesh Tris:")
    row.prop (OptimizePropGrps, "str_OptVisCol_MaxTris", text="", emboss=False)

    # Colors
    row = box.row()
    row.alignment = 'CENTER'
    row.scale_x = 0.9

    row.label(text="Min")
    row.prop (OptimizePropGrps, "enum_OptVisCol_Colors", text="", expand=True, emboss=False)
    row.label(text="Max")
    
  # endregion

# Meshes - PolyCount
class DeoPanelOptimizeMeshPolyCount (Panel) :
  bl_parent_id = "DEOS_PT_Optimize_Header"

  bl_idname = "DEOS_PT_Optimize_PolyCount"
  bl_label = "[ MESH ]    : : BY POLYCOUNT : :"
  bl_space_type = "VIEW_3D"
  bl_region_type = "UI"
  bl_options = {'DEFAULT_CLOSED'}

  def draw(self, context) :
    OptimizePropGrps = context.scene.optimizePropertyGrp
    layout = self.layout
    box = layout.box()
    rowSize = 0.5

    # Selection mode
    row = box.row()
    row.scale_y = 1.2
    row.prop(OptimizePropGrps, "enum_OptPlyCnt_SelectMode", text=" ", expand=True)             # Buttons
    # row.prop(OptimizePropGrps, "enum_OptPlyCnt_SelectMode", text="Mode", icon="OBJECT_DATA") # Dropdown
    
    # UI Collection Visible
    if OptimizePropGrps.enum_OptPlyCnt_SelectMode == 'Op3' :
      box.prop(OptimizePropGrps, "picker_OptPlyCnt_Collect", text="", icon="OUTLINER_COLLECTION")
    
    box.separator()

    # Info: Tittle
    row = box.row()
    row.alignment = 'CENTER'
    row.label(text="[ Amount of Tris to find per Mesh ]")
    
    split1 = box.row().split(factor = 0.2)
    split1.prop (OptimizePropGrps, "enum_OptPlyCnt_Action", text="")

    split2 = split1.split(factor = 0.8)
    split2.prop (OptimizePropGrps, "int_OptPlyCnt_Amount", text="")
    split2.operator ("deo.operator_empty", text="", icon="FILE_ALIAS")

    box.separator()

    # Info: Obj Tris Count action
    split = box.row().split(factor = 0.5)
    split.label(text="Mshes Found:")
    split.prop (OptimizePropGrps, "str_OptPlyCnt_ObjSel", text="", emboss=False)


#-----------------------------------------------------------------------------------------------
# REGISTRATIONS
#-----------------------------------------------------------------------------------------------

# OPERATOR EMPTY
class DeoOperatorEmpty (Operator) :
  bl_idname = "deo.operator_empty"
  bl_label = ""
  bl_description = "Empty Operator for testing purposes"
  bl_options = {'REGISTER', 'UNDO'}

  def execute(self, context) :
    self.report({'INFO'}, "Deo's Tools - Empty Operator")
    return {'FINISHED'}
  

# region 

DeosRegisterClasses = (
  DeoOperatorEmpty,

  # PROPERTIES (INIT)  
  DeosPropertyGroupOptimize, 
  # PANELS (INIT)
  DeoPanelOptimizeHeader,
  DeoPanelOptimizeMeshVisualize,
  DeoPanelOptimizeMeshPolyCount,
  # OPERATORS (TOOLs)
)


def register():
  from bpy.utils import register_class
  
  # Classes
  for myClasses in DeosRegisterClasses : 
    register_class(myClasses)
  
  # PropertyGroups
  bpy.types.Scene.optimizePropertyGrp = PointerProperty(type = DeosPropertyGroupOptimize)   
  

def unregister():  
  from bpy.utils import unregister_class  
  
  # Classes
  for myClasses in reversed(DeosRegisterClasses) : 
    unregister_class(myClasses)
  
  # PropertyGroups
  del bpy.types.Scene.optimizePropertyGrp  

if __name__ == "__main__" : register()

# endregion